import * as Dispatcher from "./Dispatcher";
import * as FluxStore from "./FluxStore";
import * as React from "react";

/**
 * A FluxContainer is used to subscribe a react component to multiple stores.
 * The stores that are used must be returned from a static `getStores()` method.
 *
 * The component receives information from the stores via state. The state
 * is generated using a static `calculateState()` method that each container must implement.
 */

/** Default options to create a Container. */
export interface RealOptions {
    /** @default true */
    pure?: boolean | undefined;
    /** @default false */
    withProps?: boolean | undefined;
    /** @default false */
    withContext?: boolean | undefined;
}

export type ComponentConstructor<TProps> = React.ComponentClass<TProps> | React.FunctionComponent<TProps>;

export type StoresList = Array<FluxStore<any>>;

export interface ComponentStatic<TProps, TState, TContext> {
    getStores(maybeProps?: TProps, maybeContext?: TContext): StoresList;
    calculateState(prevState: TState, maybeProps?: TProps, maybeContext?: TContext): TState;
}

export type Component<TProps, TState, TContext> = ComponentConstructor<TProps> & ComponentStatic<TProps, TState, TContext>;

/**
 * Create is used to transform a react class into a container
 * that updates its state when relevant stores change.
 * The provided base class must have static methods getStores() and calculateState().
 */
export function create<TProps>(base: Component<TProps, any, any>, options?: RealOptions): Component<TProps, any, any>;
export function create<TProps, TState>(base: Component<TProps, TState, any>, options?: RealOptions): Component<TProps, TState, any>;
export function create<TProps, TState, TContext>(base: Component<TProps, TState, TContext>, options?: RealOptions): Component<TProps, TState, TContext>;
export function create<TProps, TState, TContext, TStatic>(base: Component<TProps, TState, TContext> & TStatic, options?: RealOptions): Component<TProps, TState, TContext> & TStatic;

/**
 * This is a way to connect stores to a functional stateless view.
 */
export function createFunctional<TProps, TState>(
    viewFn: (props: TState) => React.ReactElement<TState>,
    getStores: (maybeProps?: TProps, maybeContext?: any) => Array<FluxStore<any>>,
    calculateState: (prevState?: TState, maybeProps?: TProps, maybeContext?: any) => TState,
    options?: RealOptions
): Component<TProps, TState, any>;
